package sun.java2d.d3d;

import sun.awt.Win32GraphicsConfig;
import sun.awt.image.SurfaceManager;
import java.awt.Image;
import sun.java2d.windows.GDIWindowSurfaceData;
import java.awt.GraphicsConfiguration;
import java.awt.image.ColorModel;
import java.awt.Component;
import sun.java2d.InvalidPipeException;
import sun.awt.windows.WComponentPeer;
import sun.java2d.SurfaceData;
import sun.awt.image.SunVolatileImage;
import sun.awt.image.VolatileSurfaceManager;

public class D3DVolatileSurfaceManager extends VolatileSurfaceManager
{
    private boolean accelerationEnabled;
    private int restoreCountdown;
    
    public D3DVolatileSurfaceManager(SunVolatileImage vImg, Object context) {
        super(vImg, context);
        int transparency = vImg.getTransparency();
        D3DGraphicsDevice d3DGraphicsDevice = (D3DGraphicsDevice)vImg.getGraphicsConfig().getDevice();
        this.accelerationEnabled = (transparency == 1 || (transparency == 3 && (d3DGraphicsDevice.isCapPresent(2) || d3DGraphicsDevice.isCapPresent(4))));
    }
    
    @Override
    protected boolean isAccelerationEnabled() {
        return this.accelerationEnabled;
    }
    
    public void setAccelerationEnabled(boolean accelerationEnabled) {
        this.accelerationEnabled = accelerationEnabled;
    }
    
    @Override
    protected SurfaceData initAcceleratedSurface() {
        Component component = this.vImg.getComponent();
        WComponentPeer wComponentPeer = (component != null) ? ((WComponentPeer)component.getPeer()) : null;
        D3DSurfaceData d3DSurfaceData;
        try {
            boolean booleanValue = false;
            if ((this.context instanceof Boolean)) {
                booleanValue = (boolean)this.context;
            }
            if (!(booleanValue == 0)) {
                d3DSurfaceData = D3DSurfaceData.createData(wComponentPeer, this.vImg);
            }
            else {
                D3DGraphicsConfig d3DGraphicsConfig = (D3DGraphicsConfig)this.vImg.getGraphicsConfig();
                ColorModel colorModel = d3DGraphicsConfig.getColorModel(this.vImg.getTransparency());
                int forcedAccelSurfaceType = this.vImg.getForcedAccelSurfaceType();
                if (forcedAccelSurfaceType == 0) {
                    forcedAccelSurfaceType = 5;
                }
                d3DSurfaceData = D3DSurfaceData.createData(d3DGraphicsConfig, this.vImg.getWidth(), this.vImg.getHeight(), colorModel, this.vImg, forcedAccelSurfaceType);
            }
        }
        catch (NullPointerException ex) {
            d3DSurfaceData = null;
        }
        catch (OutOfMemoryError outOfMemoryError) {
            d3DSurfaceData = null;
        }
        catch (InvalidPipeException ex2) {
            d3DSurfaceData = null;
        }
        return d3DSurfaceData;
    }
    
    @Override
    protected boolean isConfigValid(GraphicsConfiguration graphicsConfiguration) {
        return graphicsConfiguration == null || graphicsConfiguration == this.vImg.getGraphicsConfig();
    }
    
    private synchronized void setRestoreCountdown(int restoreCountdown) {
        this.restoreCountdown = restoreCountdown;
    }
    
    @Override
    protected void restoreAcceleratedSurface() {
        synchronized (this) {
            if (this.restoreCountdown > 0) {
                --this.restoreCountdown;
                throw new InvalidPipeException("Will attempt to restore surface  in " + this.restoreCountdown);
            }
        }
        SurfaceData initAcceleratedSurface = this.initAcceleratedSurface();
        if (initAcceleratedSurface != null) {
            this.sdAccel = initAcceleratedSurface;
            return;
        }
        throw new InvalidPipeException("could not restore surface");
    }
    
    @Override
    public SurfaceData restoreContents() {
        this.acceleratedSurfaceLost();
        return super.restoreContents();
    }
    
    static void handleVItoScreenOp(SurfaceData surfaceData, SurfaceData surfaceData2) {
        if (surfaceData instanceof D3DSurfaceData && surfaceData2 instanceof GDIWindowSurfaceData) {
            D3DSurfaceData d3DSurfaceData = (D3DSurfaceData)surfaceData;
            SurfaceManager manager = SurfaceManager.getManager((Image)d3DSurfaceData.getDestination());
            if ((manager instanceof D3DVolatileSurfaceManager)) {
                D3DVolatileSurfaceManager d3DVolatileSurfaceManager = (D3DVolatileSurfaceManager)manager;
                if (d3DVolatileSurfaceManager != null) {
                    d3DSurfaceData.setSurfaceLost(true);
                    WComponentPeer peer = ((GDIWindowSurfaceData)surfaceData2).getPeer();
                    if (D3DScreenUpdateManager.canUseD3DOnScreen(peer, (Win32GraphicsConfig)peer.getGraphicsConfiguration(), peer.getBackBuffersNum())) {
                        d3DVolatileSurfaceManager.setRestoreCountdown(10);
                    }
                    else {
                        d3DVolatileSurfaceManager.setAccelerationEnabled(false);
                    }
                }
            }
        }
    }
    
    @Override
    public void initContents() {
        if (this.vImg.getForcedAccelSurfaceType() != 3) {
            super.initContents();
        }
    }
}
